/*=================================================================*/
/** @file D2UnitStruct.h
 *  @brief Diablo II Unit Structures definitions.
 *
 *  (c) January 2002 - Joel Falcou for The Phrozen Keep.
 */
/*=================================================================*/

#ifndef __D2UNITSTRUCT_H__INCLUDED__
#define __D2UNITSTRUCT_H__INCLUDED__

/*=================================================================*/
/*    Unit Types enumerations.                                     */
/*=================================================================*/

enum {  UNIT_PLAYER = 0,
        UNIT_MONSTER,
	    UNIT_OBJECT,
	    UNIT_MISSILE,
	    UNIT_ITEM,
	    UNIT_ROOMTILE
     };

/*=================================================================*/
/*    Preset Unit Structures (used by DrlgRoom2).                  */
/*=================================================================*/

struct PresetUnit 
{                                   //Offset from Code
	DWORD           unitType;       //+00
	DWORD           txtFileNo;      //+04
	DWORD           unknown1[1];    //Unkonwn Data at +08
	DWORD           xPos;           //+0c
	DWORD           yPos;           //+10
	DWORD           unknown2[2];    //Unknown data at +14
	PresetUnit*     pNext;          //+1c
};


/*=================================================================*/
/*   Unit Data Structure.                                          */
/*=================================================================*/

struct Unit 
{								//Offset from Code.
	DWORD		nUnitType;			//+00 Unit Type
	DWORD		nUnitId;			//+04 Unit ID
    DWORD*		unknown_1;			//+08 ptGame + 1C
    DWORD		nUnitUnid;			//+0C Unique ID
	DWORD		nAnimMode;			//+10 Current animation mode

    union 
    {
		PlayerData*     pPlayerData;
		MonsterData*    pMonsterData;
		ObjectData*     pObjectData;
		ItemData*       pItemData;
	};				                //+14 ptUnitData

    DWORD		nAct;		        //+18 Act
	DWORD		ptAct;				//+1C ptAct
	DWORD		unknown_3;			//+20
	DWORD		unknown_4;			//+24
	DWORD		unknown_5;			//+28
	Path*		hPath;				//+2C
	DWORD		unknown_7;			//+30
	DWORD		unknown_8;			//+34
	Path*		ptPos;				//+38 ptPos
	DWORD		unknown_10;			//+3C
	DWORD		unknown_11;			//+40
	DWORD		unknown_12;			//+44
	DWORD		unknown_13;			//+48
	DWORD		unknown_14;			//+4C
	DWORD		nAnimData;			//+50 AnimData.D2 record address for current mode
	DWORD		unknown_16;			//+54
	DWORD		unknown_17;			//+58
/*	StatsList*	ptStatsList;		//+5C ptStatsList
	Inventory*	ptInventory;		//+60 ptInventory
    DWORD		unknown_19;			//+64
	DWORD		unknown_20;			//+68
	WORD		unknown_21_A;		//+6C
	WORD		unknown_21_B;		//+6E Cube puts 4 here
	DWORD		unknown_22;			//+70 Cube puts the units ptUnit of itself here
	DWORD		unknown_23;			//+74 0 at start
	DWORD		unknown_24;			//+78
	DWORD		unknown_25;			//+7C
	D2Game*		ptGame;				//+80 ptGame
	DWORD		unknown_26;			//+84
	DWORD		unknown_27;			//+88
	DWORD		unknown_28;			//+8C
	DWORD		unknown_29;			//+90
	DWORD		unknown_30;			//+94
	DWORD		unknown_31;			//+98
	DWORD		unknown_32;			//+9C
	DWORD		unknown_33;			//+A0
	DWORD		unknown_34;			//+A4
	D2Info*		ptInfo;				//+A8 ptInfo
	DWORD		unknown_35;			//+AC
	DWORD		unknown_36;			//+B0
	DWORD		unknown_37;			//+B4
	DWORD		unknown_38;			//+B8
	DWORD		unknown_39;			//+BC
	DWORD		unknown_40;			//+C0
	DWORD		nFlags;				//+C4 Flags
	DWORD		nLODFlag;			//+C8 LODFlag
	DWORD		unknown_41;			//+CC
	DWORD		nClientNum;			//+D0 Client# (0B at start)
*/
};

/*=================================================================*/
/*   Unit Specific Data Structure.                                 */
/*=================================================================*/

struct PlayerData 
{
};

struct MonsterData 
{                       // Offset from Code
};

struct ObjectData 
{                       // Offset from Code
};

struct ItemData 
{                       // Offset from Code
	DWORD nQuality;     //+00
};

/*=================================================================*/
/*   Item Location Structure.                                     */
/*=================================================================*/

struct LocInfo
{                       //Offset from Code
    DWORD noneLoc;      //+00
    DWORD headLoc;      //+04
    DWORD neckLoc;      //+08
    DWORD torsoLoc;     //+0c
    DWORD rightArmLoc;  //+10
    DWORD leftArmLoc;   //+14
    DWORD rightRingLoc; //+18
    DWORD leftRingLoc;  //+1c
    DWORD beltLoc;      //+20
    DWORD feetLoc;      //+24
    DWORD glovesLoc;    //+28
};

/*=================================================================*/
/*   Inventory Node Structure.                                     */
/*=================================================================*/

struct Node
{                   //Offset from Code.
    DWORD null;     //+00
    ItemData* pItem;    //+04
    DWORD unID;     //+08
    DWORD page;     //+0c
    Node* nextNode; //+10
};

/*=================================================================*/
/*   Inventory Structure.                                          */
/*=================================================================*/

struct UnitInventory 
{                               //Offset from Code.
    DWORD       gameRelated;    //+00
    DWORD       unit;           //+04
    DWORD       unID[2];        //+0c
    DWORD       unknown1[2];    //+14
    LocInfo*    ptLocInfo;      //+1c
    DWORD       unknown[2];     //Unknown Data at +20
    Node*       ptNode;         //+28
};

/*=================================================================*/
/*  Skill Structure.                                               */
/*=================================================================*/

struct Skills
{                           //Offset from Code.
    DWORD   skillTxt;       //+00
    Skills* pNextSkill;     //+04
    DWORD   mode;           //+08
    DWORD   unknown1[3];    //+0C
    DWORD   targetInfo;     //+18
    DWORD   targetType;     //+1c
    DWORD   targetUNID;     //+20
    DWORD   unknown2;       //+24
    DWORD   slvl;           //+28
    DWORD   slvlBonus;      //+2c
    DWORD   unknown3;       //+30
    DWORD   state;          //+34
};

/*=================================================================*/
/*  Skills Info Structure.                                         */
/*=================================================================*/

struct D2Info
{                           //Offset from Code.
    DWORD   gameRelated;    //+00
    Skills* ptSkill;        //+04
    Skills* ptLeftSkill;    //+08
    Skills* ptRightSkill;   //+0c
    Skills* ptCurrentSkill; //+10
};

struct Stats  {};
struct Combat {};
struct D2Game {};


/*=================================================================*/
/*   Unit Related Helper Functions.                                */
/*=================================================================*/

D2Game* getGamePtrFromUnit( UnitAny* pUnit );


#endif