/*==============================================*/
/** @file d2functions.hpp 
    @brief Exported Diablo II DLL functions.

    This file implements some common and useful
    function related to some Diablo II mechanisms.

 */ 
/*==============================================*/

#ifndef __D2FUNCTIONS_HPP__INCLUDED__
#define __D2FUNCTIONS_HPP__INCLUDED__

#include <windows.h>
#include "error.hpp"
#include "D2DLL.hpp"
#include "D2Struct.hpp"
#include "D2UnitStruct.h"
#include "common.hpp"

extern DWORD FogOffset;
extern DWORD GfxOffset;
extern DWORD CommonOffset;

#define STDCALL		__stdcall
#define FASTCALL	__fastcall



/*==============================================*/
/** 
 *  @brief stdcall function pointer declaration.
 *
 *  Declares the type for a STDCALL function pointer
 *  called N with arguments list A adn returning R.
 *
 */
/*==============================================*/

#define STDCALL_FUNCTION( R, N, A )  typedef R (_stdcall *N) A

/*==============================================*/
/**
 *  @brief fastcall function pointer declaration.
 *
 *  Declares the type for a FASTCALL function pointer
 *  called N with arguments list A adn returning R.
 *
 */
/*==============================================*/

#define FASTCALL_FUNCTION( R, N, A ) typedef R (_fastcall *N) A

/*=================================================================*/
/* D2Common functions prototypes                                   */
/*=================================================================*/

STDCALL_FUNCTION( DWORD      , D2Common29EB    , (Unit* ptItem, DWORD TypeID));
STDCALL_FUNCTION( BYTE       , D2Common29C1    , (Unit* ptItem));

//String Stuff
STDCALL_FUNCTION( void       , D2Fog10023      , ( const char* ptMessage, const char* ptLocation, DWORD line ));
STDCALL_FUNCTION( HWND		 , D2Gfx10027	   , ());
FASTCALL_FUNCTION( void*	 , D2Fog10042	   , ( DWORD dwMemSize, LPCSTR lpszErrFile, DWORD ErrLine, DWORD Zero));
FASTCALL_FUNCTION( void*	 , D2Fog10043	   , ( void* ptMemLoc, LPCSTR lpszErrFile, DWORD ErrLine, DWORD Zero));


/*=================================================================*/
/* D2Common functions pointer.                                     */
/*=================================================================*/

static D2Fog10023       FogAssert                  = (D2Fog10023)       (FogOffset + 0xED30);	// 1.10
static D2Fog10042		FogMemAlloc				   = (D2Fog10042)	    (FogOffset + 0x8F50);	// 1.10
static D2Fog10043		FogMemDeAlloc			   = (D2Fog10043)	    (FogOffset + 0x8F90);	// 1.10
static D2Gfx10027		D2GetWindowHandle		   = (D2Gfx10027)	    (GfxOffset + 0x49C0);	// 1.10

static D2Common29EB     D2CheckMatch               = (D2Common29EB)     (CommonOffset + 0x5DCE0); // 1.10
static D2Common29C1     D2GetBodyLoc               = (D2Common29C1)     (CommonOffset + 0x58430); // 1.10

#endif